/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.dispatch;

import biz.hammurapi.util.PoliteVisitor;

/**
 * Dispatches visited objects to visit- and leave- dispatchers.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class DispatchingVisitor implements PoliteVisitor {

	private Dispatcher visitDispatcher;
	private Dispatcher leaveDispatcher;

	/**
	 * Creates DispatchingVisitor
	 * @param visitDispatcher
	 * @param leaveDispatcher
	 */
	public DispatchingVisitor(Dispatcher visitDispatcher, Dispatcher leaveDispatcher) {
		super();
		this.visitDispatcher=visitDispatcher;
		this.leaveDispatcher=leaveDispatcher;
	}

	/**
	 * Passes argument to visit dispatcher.
	 */
	public void leave(Object arg) {
		if (leaveDispatcher!=null) {
			leaveDispatcher.dispatch(arg);
		}
	}

	/**
	 * Passes argument to leave dispatcher.
	 */
	public boolean visit(Object arg) {
		if (visitDispatcher!=null) {
			visitDispatcher.dispatch(arg);
		}
		return true;
	}

}
