/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.dispatch;

/**
 * Handler to invoke methods with one argument.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public interface InvocationHandler {

	/**
	 * Invokes target method. Passes returned value(s) to result consumer.
	 * The target "method" might be invoked multiple times, e.g. in composite
	 * invocation handler scenario.
	 * @param parameter
	 * @param resultConsumer
	 * @throws Throwable
	 */
	void invoke(Object arg, ResultConsumer resultConsumer) throws Throwable;
		
	/**
	 * This method is used to build invocation network.
	 * @return Invocation parameter type. Null if parameter type is unknown.
	 */
	Class getParameterType();
	
	/**
	 * This method is used for backward reasoning. 
	 * @return Types of facts produced (returned or posted) by this handler.
	 */
	Class[] getFactTypes();
}
