/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import biz.hammurapi.config.ComponentBase;
import biz.hammurapi.config.ConfigurationException;
import biz.hammurapi.dispatch.InvocationTarget;

public abstract class AbstractRule extends ComponentBase implements InvocationTarget, Constants {

	private String name;
	private String description;

	public String getName() {
		return name;
	}

	/**
	 * This method is invoked by rule container to set rule name.
	 * Rule name is used to retrieve collections from collection 
	 * manager.
	 * @param name
	 */
	public void setName(String name) {
		this.name=name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String toString() {
		return "["+getClass().getName()+"] "+getName()+" - "+getDescription();
	}

	public abstract Collection getRemoveHandlers();
	
	public void start() throws ConfigurationException {
		this.collectionManager=(CollectionManager) get("/"+COLLECTION_MANAGER);
		if (getName()==null) {
			throw new ConfigurationException("Rule name must not be null");
		}		
	}
	
	public void stop() {
		// Override this method to release resources.
	}			
	
	private Collection acquiredCollections = new ArrayList();
	
	/**
	 * Convenience method to retrieve collection from the collection manager.
	 * Collections retrieved through this method are cleared in reset();
	 * @param setName
	 * @return
	 */
	protected Collection getCollection(String collectionName, Object lock) {
		Collection ret = collectionManager.get(name, collectionName, lock);
		acquiredCollections.add(ret);
		return ret;
	}
		
	private CollectionManager collectionManager;

	/**
	 * Invoked in rule session reset() method. Cleans acquired collections.
	 * Subclasses can put additional cleanup logic here.
	 */
	public void reset() {
		// Rules session clears collection manager. This functionality
		// is a duplicate of reset() functionality in rules session, 
		// for a case if rules are ever reset on individual basis.
		Iterator cit = acquiredCollections.iterator();
		while (cit.hasNext()) {
			((Collection) cit.next()).clear();
		}
	}
	
}
