/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.util.Collection;

/**
 * Implementations manage collections for rules.
 * Rules itself shall not maintain collections of 
 * facts and objects, but use CollectionManager.
 * 
 * @author Pavel Vlasov
 * @version ${Revision}
 */
public interface CollectionManager extends Negatable {
	
	/**
	 * Retrieves collection from the manager.
	 * If a collection for this name doesn't exist it 
	 * should be created.
	 * @param ruleName Name of the rule this collection belongs to.
	 * @param collectionName Name of the collection withing the rule.
	 * @return Collection, never null.
	 * @param lock Collection monitor for synchronization.
	 */
	Collection get(String ruleName, String collectionName, Object lock);
	

	/**
	 * Clears all collections.
	 */
	void clear();	
}
