/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Map;

import biz.hammurapi.config.ConfigurationException;

public class FileObjectStorage implements ObjectStorage {
	private Map objects;

	public void put(String key, Object value) {
		objects.put(key, value);

	}

	public void start() throws ConfigurationException {
		if (file==null) {
			throw new ConfigurationException("Storage file name is not set");
		}
		
		if (file.exists()) {
			if (file.isFile()) {
				try {
					ObjectInputStream ois=new ObjectInputStream(new FileInputStream(file));
					objects=(Map) ois.readObject();
					ois.close();						
				} catch (ClassNotFoundException e) {
					throw new ConfigurationException("Could not load objects from: "+file.getAbsolutePath()+", class not found "+e, e);						
				} catch (FileNotFoundException e) {
					throw new ConfigurationException("Storage file does not exist: "+file.getAbsolutePath(), e);
				} catch (IOException e) {
					throw new ConfigurationException("Could not load objects from: "+file.getAbsolutePath(), e);
				}
			} else {
				throw new ConfigurationException("Not a file: "+file.getAbsolutePath());
			}
		}
	}

	public void stop() throws ConfigurationException {
		try {
			ObjectOutputStream oos=new ObjectOutputStream(new FileOutputStream(file));
			oos.writeObject(objects);
			oos.close();
		} catch (FileNotFoundException e) {
			throw new ConfigurationException("Collections file does not exist: "+file.getAbsolutePath(), e);
		} catch (IOException e) {
			throw new ConfigurationException("Could not load collections from: "+file.getAbsolutePath(), e);
		}			
	}
	
	private File file;
	
	public void setFile(File file) {
		this.file = file;
	}

	public Object get(String name) {
		return objects.get(name);
	}
}
