/**
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz 
 */
package biz.hammurapi.rules;

import java.util.Collection;

import javax.rules.Handle;

/**
 * Handle manager - creates and maintains handle -> object
 * mappings.
 * @author Pavel Vlasov
 * @version ${Revision}
 */
public interface HandleManager extends Negatable {
	
	/**
	 * Establishes Handle -> Object mapping.
	 * @param object
	 * @return
	 */
	Handle addObject(Object object);
	
	/**
	 * Retrieves object by handle
	 * @param handle
	 * @return
	 */
	Object getObject(Handle handle);
	
	/**
	 * @return All objects with handles.
	 */
	Collection getObjects();
	
	/**
	 * @return All handles.
	 */
	Collection getHandles();
	
	/**
	 * Removes object entry by key.
	 * @param handle
	 */
	void remove(Handle handle);
	
	/**
	 * Removes object entry by value;
	 * @param obj
	 */
	void remove(Object obj);	
	
	/**
	 * @param handle
	 * @return True there is a mapping with given handle in the manager.
	 */
	boolean contains(Handle handle);
	
	/**
	 * Rebinds handle to another object
	 * @param handle
	 * @param object
	 */
	void rebind(Handle handle, Object object);

	/**
	 * Removes all mappings.
	 */
	void clear();
}
