/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.util.Collection;

/**
 * This interface is a facade for object dispatchers.
 * Rule containers shall implement this interface.
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public interface KnowledgeBase {
	
	/**
	 * Interface for dispatching commands to knowledge bases.
	 * @author Pavel
	 */
	public interface KnowledgeBaseCommand {

		/**
		 * Knowledge base invokes this method when 
		 * instance of command is dispatched to it.
		 * @param knowledgeBase
		 */
		void execute(KnowledgeBase knowledgeBase);
	}
	
	/**
	 * Adds object to the knowledge base.
	 * @param obj
	 */
	void add(Object obj);
	
	/**
	 * Removes object from the knowledge base
	 * and all conclusions based on this object.
	 * @param obj
	 */
	void remove(Object obj);
	
	/**
	 * Executes rules
	 */
	void executeRules();

	/**
	 * @return Collection of rule instances in the knowledge base.
	 */
	Collection getRules();
	
	/**
	 * Resets rules state (if any).
	 */
	void reset();

}
