/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class KnowledgeMaximizingCompactor implements KnowledgeCompactor {

	public KnowledgeMaximizingCompactor() {
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * This method preprocesses objects before returning them from getObjects() method.
	 * This implementation "maximizes knowledge" i.e. removes conclusions for which there are
	 * superceding conslusions. E.g. Father(John, Peter) supercedes Parent(John, Peter) and if 
	 * both these conclusions are in the source collection then Parent(John, Peter) will not be 
	 * included in the target collection. If there are two equal conclusions then their derivations
	 * are merged and only one conclusion is retained.
	 * @param results
	 * @param finalCollection
	 */
	public List compact(List results) {
		List ret=new ArrayList();
		Iterator it=results.iterator();
		Z: while (it.hasNext()) {
			Object newFact=it.next();
			if (newFact instanceof Supercedable) {
				Iterator fit=ret.iterator();
				while (fit.hasNext()) {
					Object existingFact=fit.next();
					if (existingFact instanceof Supercedable) {
						Supercedable ec=(Supercedable) existingFact;
						Supercedable nc=(Supercedable) newFact;
						
						if (ec instanceof Conclusion && nc instanceof Conclusion && ec.equals(nc)) {
							((Conclusion) ec).mergeDerivations((Conclusion) nc);
							continue Z;
						}
						
						if (ec.supercedes(nc)) { // Existing fact is more specific than new fact.
							continue Z;
						}
						
						if (nc.supercedes(ec)) {
							fit.remove();
						}
					}
				}
			}
				
			ret.add(newFact);
		}
		return ret;
	}
		
}
