/*
 * hammurapi-rules @mesopotamia.version@
 * Hammurapi rules engine. 
 * Copyright (C) 2005  Hammurapi Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * URL: http://http://www.hammurapi.biz
 * e-Mail: support@hammurapi.biz
 */
package biz.hammurapi.rules;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * This class "maximizes knowledge", which means when more specific (superceding) 
 * conclusions are added to the set less specific are removed. Conclusions less specific than present in 
 * the set are not added at all. 
 * @author Pavel Vlasov
 * @revision $Revision$
 */
public class KnowledgeMaximizingSet implements Set {
	private Collection master;

	/**
	 * Adds object if collection doesn't contain equal or superceding object.
	 */
	public boolean add(Object arg) {
		if (arg instanceof Supercedable) {
			Supercedable nc=(Supercedable) arg;
			Iterator it=master.iterator();
			while (it.hasNext()) {
				Object o=it.next();
				if (o instanceof Supercedable) {
					Supercedable ec=(Supercedable) o;
					if (ec.equals(nc) || ec.supercedes(nc)) {
						return false;
					}
					
					if (nc.supercedes(ec)) {
						it.remove();
						break; // There can be only one existing object in the collection superceded by this one.
					}
				}
			}
			return master.add(arg);
		} 
		
		return master.add(arg);
	}

	public boolean addAll(Collection arg) {
		boolean ret=false;
		Iterator it=arg.iterator();
		while (it.hasNext()) {
			ret = ret || add(it.next());
		}
		return ret;
	}

	public void clear() {
		master.clear();
	}

	public boolean contains(Object arg0) {
		return master.contains(arg0);
	}

	public boolean containsAll(Collection arg0) {
		return master.containsAll(arg0);
	}

	public boolean equals(Object arg0) {
		return master.equals(arg0);
	}

	public int hashCode() {
		return master.hashCode();
	}

	public boolean isEmpty() {
		return master.isEmpty();
	}

	public Iterator iterator() {
		return master.iterator();
	}

	public boolean remove(Object arg0) {
		return master.remove(arg0);
	}

	public boolean removeAll(Collection arg0) {
		return master.removeAll(arg0);
	}

	public boolean retainAll(Collection arg0) {
		return master.retainAll(arg0);
	}

	public int size() {
		return master.size();
	}

	public Object[] toArray() {
		return master.toArray();
	}

	public Object[] toArray(Object[] arg0) {
		return master.toArray(arg0);
	}

	/**
	 * @param master Master collection
	 */
	public KnowledgeMaximizingSet(Collection master) {
		this.master=master;
	}

	/**
	 * Constructs new KnowledgeMaximizingSet with java.util.HashSet as master.
	 */
	public KnowledgeMaximizingSet() {
		this(new HashSet());
	}

}
