package biz.hammurapi.rules.backwardreasoning;

import java.util.logging.Logger;

import biz.hammurapi.rules.Conclusion;

/**
 * This implementation eliminates facts duplicates. Derivations
 * of identical conclusions are merged.
 * @author Pavel
 *
 */
public class NoDuplicatesBackwardReasoningRulesContainer extends
		BackwardReasoningRulesContainer {
	
	private static final Logger logger = Logger.getLogger(NoDuplicatesBackwardReasoningRulesContainer.class.getName());

	protected Accumulator createAccumulator(Class type) {
		return new Accumulator(type) {
			
			/**
			 * Runs along the chain and compares new element
			 * with existing before adding.
			 */
			public synchronized void add(Object obj) {
				if (obj == null) {
					throw new NullPointerException(this.getClass().getName()+" does not support null elements.");
				}				
				
				for (Link link = first.peekNext(); link!=null && link!=END; link = link.peekNext()) {
					if (obj.equals(link.getValue())) {
						logger.finest("Accumulator "+this+" already contains "+obj);
						if (obj instanceof Conclusion) {
							((Conclusion) link.getValue()).mergeDerivations((Conclusion) obj);
						}
						return;
					}
				}
				super.add(obj);
			}
			
			protected void onBlock() {
				logger.finest("Accumulator "+this+" enters blocking state, starting anti-blocking measures");
				onAccumulatorBlock();
			}
			
		};
	}
}
